/***************************************************************************
 *   Copyright (C) 2003 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the osgGtk library.                              *
 *                                                                         *
 *   The osgGtk library is free software; you can redistribute it and/or   *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The osgGtk library is distributed in the hope that it will be         *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

// Of course, we need the gtk headers
#include <gtk/gtk.h>

// We'll need this to load the command line argument
#include <osgDB/ReadFile>

// And we'll need this for the Gtk OSG viewer
#include <osgGtk/ViewerGtk.h>

// This is global so that the callbacks can see it as well
// Alternatively, we could use the data element of the callback,
// but making this global simplifies the example.
osgViewer::ViewerGtk* viewer = NULL;

// This is also global so the callback function can check to see the
// state of the toggle button
GtkToggleButton* fps_button;

// Callback to handle mouse button events
bool on_button_event(GdkEventButton* event);

// Callback that loads a selected scene as the root node of the scenegraph
void on_open_file_clicked();

// Callback that runs the viewer a specific number of frames
void on_run_frames_button_clicked();

// Callback that starts/stops the viewer
void on_fps_toggled();

int main( int argc, char** argv )
{
  // We'll set a basic size of 800x600, but the user can resize larger if desired
  // (but not smaller)
  int width=800, height=600;
  
  // This is some gtk housekeeping... we need the main loop and we also need to
  // initialize the OpenGL system
  gtk_init( &argc, &argv );
  gtk_gl_init( &argc, &argv );

  // Now, let's create our viewer using ViewerGtk so we can use the gtk
  // convenience method later
  //
  // We'll also set desired frame rate of 60 fps and load the first command line
  // argument as the scene
  osgViewer::ViewerGtk viewer;
  viewer.set_fps(60);

  if ( argc >= 2 ) {
    osg::ref_ptr<osg::Node> model = osgDB::readNodeFile(argv[1]);
    if ( model.valid() ) viewer.setSceneData(model.get());
  }

  // Now we'll setup the viewer in a gtk window using the setup_viewer_in_gtk_window()
  // convenience method
  //
  // The return type is a Gtk widget that we can use anywhere we want
  osgViewer::GraphicsWindowGtk* gw = viewer.setup_viewer_in_gtk_window(width,height);

  // All Gtk apps need a toplevel window
  //
  // We'll also set the title, add our widget created by the viewer and make sure
  // it's visible
  GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "osgviewerGtk");
  gtk_container_add(GTK_CONTAINER(window), gw->gtk_widget());
  g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_main_quit), 0 );
  gtk_widget_show_all(window);

  // Now, we'll call run on the viewer AFTER we have the graphics window inside a
  // top-level window
  viewer.run();
  
  // This is boilerplate for all gtk apps... start the gtk main loop
  gtk_main();

  // If we got here, gtk exited it's main loop, so we'll exit cleanly
  return 0;
}
